@@ -206,7 +206,7 @@ file_magic llvm::identify_magic(StringRef Magic) {
 }
 
 std::error_code llvm::identify_magic(const Twine &Path, file_magic &Result) {
-  auto FileOrError = MemoryBuffer::getFile(Path);
+  auto FileOrError = MemoryBuffer::getFile(Path, -1LL, false);
   if (!FileOrError)
     return FileOrError.getError();
 
