@@ -16,13 +16,6 @@
 
 using namespace llvm;
 
-#ifndef LLVM_BUILD_GLOBAL_ISEL
-
-void llvm::initializeGlobalISel(PassRegistry &Registry) {
-}
-
-#else
-
 void llvm::initializeGlobalISel(PassRegistry &Registry) {
   initializeIRTranslatorPass(Registry);
   initializeLegalizerPass(Registry);
@@ -30,4 +23,3 @@ void llvm::initializeGlobalISel(PassRegistry &Registry) {
   initializeRegBankSelectPass(Registry);
   initializeInstructionSelectPass(Registry);
 }
-#endif // LLVM_BUILD_GLOBAL_ISEL
